<?php
/**
 * @file
 * Hooks and callbacks to handle page2images field.
 */

/**
 * Implements hook_field_info().
 */
function page2images_field_info() {
  return array(
    'page2images' => array(
      'label' => t('Page2Images'),
      'description' => t('Provides a field to store an URL and an image generated by Page2Images.'),
      'settings' => array(
        'uri_scheme' => variable_get('file_default_scheme', 'public'),
        'default_image' => 0,
      ),
      'instance_settings' => array(
        'p2i_device' => 6,
        'p2i_size' => '',
        'p2i_screen' => '',
        'p2i_fullpage' => 1,
        'p2i_imageformat' => 'png',
        'p2i_quality_jpg' => 90,
        'p2i_quality_png' => 80,
        'alt_field' => 0,
        'title_field' => 0,
        'uri_scheme' => variable_get('file_default_scheme', 'public'),
        'default_image' => 0,
        'file_directory' => '',
      ),
      'default_widget' => 'page2images',
      'default_formatter' => 'page2images_rest',
    ),
  );
}

/**
 * Implements hook_field_settings_form().
 */
function page2images_field_settings_form($field, $instance) {
  $defaults = field_info_field_settings($field['type']);
  $settings = array_merge($defaults, $field['settings']);

  $scheme_options = array();
  foreach (file_get_stream_wrappers(STREAM_WRAPPERS_WRITE_VISIBLE) as $scheme => $stream_wrapper) {
    $scheme_options[$scheme] = $stream_wrapper['name'];
  }

  $form['uri_scheme'] = array(
    '#type' => 'radios',
    '#title' => t('Upload destination'),
    '#description' => t('Select where the final files should be stored. Private file storage has significantly more overhead than public files, but allows restricted access to files within this field.'),
    '#options' => $scheme_options,
    '#default_value' => $settings['uri_scheme'],
  );

  $form['default_image'] = array(
    '#type' => 'managed_file',
    '#title' => t('Default image'),
    '#description' => t('If no image, this image will be shown on display.'),
    '#default_value' => $field['settings']['default_image'],
    '#upload_location' => $settings['uri_scheme'] . '://default_images/',
  );

  return $form;
}

/**
 * Implements hook_field_instance_settings_form().
 */
function page2images_field_instance_settings_form($field, $instance) {
  $settings_defaults = $instance['settings'];
  $settings_saved = isset($instance['settings']['wrapper']) ? $instance['settings']['wrapper'] : array();
  $settings = array_merge($settings_defaults, $settings_saved);

  $form['wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t('Page2Images field instance settings'),
    '#weight' => 10,
  );

  $form['wrapper']['p2i_device'] = array(
    '#type' => 'select',
    '#title' => t('Device'),
    '#description' => t('Page2Images can generate the screenshot for these smart phones and desktops.'),
    '#options' => array(
      0 => 'iPhone4',
      1 => 'iPhone5',
      2 => 'Android',
      3 => 'WinPhone',
      4 => 'iPad',
      5 => 'Android Pad',
      6 => 'Desktop',
    ),
    '#default_value' => $settings['p2i_device'],
  );

  $p2i_size = explode('x', $settings['p2i_size']) + array('', '');

  $desc_p2i_size = t('Page2Images predefined the size for those devices for most usage:');
  $desc_p2i_size .= '<br />';
  $desc_p2i_size .= '<ul>';
  $desc_p2i_size .= '<li>' . t('Mobile Phones: 320x480') . '</li>';
  $desc_p2i_size .= '<li>' . t('Tablets: 512x384') . '</li>';
  $desc_p2i_size .= '<li>' . t('Desktop: 640x400') . '</li>';
  $desc_p2i_size .= '</ul>';
  $desc_p2i_size .= '<br />';
  $desc_p2i_size .= '<b>' . t('Advanced information:') . '</b>';
  $desc_p2i_size .= '<ul>';
  $desc_p2i_size .= '<li>' . t('If you only know the fixed width or height of the images you want to get, you can just pass the width or height to us by following format: 512x0 or 0x512. We will resize the image to fit that width/height and make sure it is the biggest image we can generate.') . '</li>';
  $desc_p2i_size .= '<li>' . t('The screenshot image will be resized to fit the width and height without stretching.') . '</li>';
  $desc_p2i_size .= '</ul>';

  $form['wrapper']['p2i_size'] = array(
    '#type' => 'item',
    '#title' => t('Fixed width or/and height of the image'),
    '#description' => $desc_p2i_size,
    '#element_validate' => array('_page2images_field_p2i_size_validate'),
    '#field_prefix' => '<div class="container-inline">',
    '#field_suffix' => '</div>',
  );

  $form['wrapper']['p2i_size']['x'] = array(
    '#type' => 'textfield',
    '#title' => t('Width'),
    '#title_display' => 'invisible',
    '#default_value' => $p2i_size[0],
    '#size' => 5,
    '#maxlength' => 5,
    '#field_suffix' => ' x ',
  );

  $form['wrapper']['p2i_size']['y'] = array(
    '#type' => 'textfield',
    '#title' => t('Height'),
    '#title_display' => 'invisible',
    '#default_value' => $p2i_size[1],
    '#size' => 5,
    '#maxlength' => 5,
    '#field_suffix' => ' ' . t('pixels'),
  );

  $p2i_screen = explode('x', $settings['p2i_screen']) + array('', '');

  $desc_p2i_screen = t('Page2Images predefined the size for those devices for most usage:');
  $desc_p2i_screen .= '<br />';
  $desc_p2i_screen .= '<ul>';
  $desc_p2i_screen .= '<li>' . t('Mobile: 320x480') . '</li>';
  $desc_p2i_screen .= '<li>' . t('Mobile iPhone5: 320x500') . '</li>';
  $desc_p2i_screen .= '<li>' . t('Tablets: 1024x768') . '</li>';
  $desc_p2i_screen .= '<li>' . t('Desktop: 1280x800') . '</li>';
  $desc_p2i_screen .= '</ul>';
  $desc_p2i_screen .= '<br />';
  $desc_p2i_screen .= '<b>' . t('Advanced information:') . '</b>';
  $desc_p2i_screen .= '<ul>';
  $desc_p2i_screen .= '<li>' . t('The screen size should not be less than image size.') . '</li>';
  $desc_p2i_screen .= '</ul>';

  $form['wrapper']['p2i_screen'] = array(
    '#type' => 'item',
    '#title' => t('The screen size of device you want to simulate'),
    '#description' => $desc_p2i_screen,
    '#element_validate' => array('_page2images_field_p2i_screen_validate'),
    '#field_prefix' => '<div class="container-inline">',
    '#field_suffix' => '</div>',
  );

  $form['wrapper']['p2i_screen']['x'] = array(
    '#type' => 'textfield',
    '#title' => t('Width'),
    '#title_display' => 'invisible',
    '#default_value' => $p2i_screen[0],
    '#size' => 5,
    '#maxlength' => 5,
    '#field_suffix' => ' x ',
  );

  $form['wrapper']['p2i_screen']['y'] = array(
    '#type' => 'textfield',
    '#title' => t('Height'),
    '#title_display' => 'invisible',
    '#default_value' => $p2i_screen[1],
    '#size' => 5,
    '#maxlength' => 5,
    '#field_suffix' => ' ' . t('pixels'),
  );

  $form['wrapper']['p2i_fullpage'] = array(
    '#type' => 'checkbox',
    '#title' => t('Full page or just the screen area.'),
    '#description' => t('If the checkbox is checked, Page2Images will take the full page screenshot. If the checkbox is not checked, Page2Images will take the screen area only.'),
    '#default_value' => $settings['p2i_fullpage'],
  );

  $form['wrapper']['p2i_imageformat'] = array(
    '#type' => 'select',
    '#title' => t('Image Format'),
    '#description' => t('JPG file is smaller than PNG. But PNG quaility is much better than JPG.'),
    '#options' => array(
      'png' => 'PNG',
      'jpg' => 'JPG',
    ),
    '#default_value' => $settings['p2i_imageformat'],
  );

  $form['wrapper']['p2i_quality_jpg'] = array(
    '#type' => 'textfield',
    '#title' => t('The quality of the image (JPG)'),
    '#description' => t('By default, JPG is 90. JPG: you can change it from 80 to 95 as needed.'),
    '#default_value' => $settings['p2i_quality_jpg'],
    '#field_suffix' => ' %',
    '#size' => 3,
  );

  $form['wrapper']['p2i_quality_png'] = array(
    '#type' => 'textfield',
    '#title' => t('The quality of the image (PNG)'),
    '#description' => t('By default, PNG is 80. PNG: you can change it from 70 to 85 as needed.'),
    '#default_value' => $settings['p2i_quality_png'],
    '#field_suffix' => ' %',
    '#size' => 3,
  );

  // Add title and alt configuration options.
  $form['wrapper']['alt_field'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable <em>Alt</em> field'),
    '#default_value' => $settings['alt_field'],
    '#description' => t('The alt attribute may be used by search engines, screen readers, and when the image cannot be loaded.'),
    '#weight' => 10,
  );

  $form['wrapper']['title_field'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable <em>Title</em> field'),
    '#default_value' => $settings['title_field'],
    '#description' => t('The title attribute is used as a tooltip when the mouse hovers over the image.'),
    '#weight' => 11,
  );

  $form['wrapper']['file_directory'] = array(
    '#type' => 'textfield',
    '#title' => t('File directory'),
    '#description' => t('Optional subdirectory within the upload destination where files will be stored. Do not include preceding or trailing slashes.'),
    '#default_value' => $settings['file_directory'],
    '#element_validate' => array('_page2images_file_directory_validate'),
    '#weight' => 12,
  );

  $form['wrapper']['default_image'] = array(
    '#type' => 'managed_file',
    '#title' => t('Default image'),
    '#description' => t('If no image, this image will be shown on display.'),
    '#default_value' => $settings['default_image'],
    '#upload_location' => $settings['uri_scheme'] . '://default_images/',
    '#weight' => 13,
  );

  return $form;
}

/**
 * Implements hook_field_is_empty().
 */
function page2images_field_is_empty($item, $field) {
  return empty($item['url']);
}

/**
 * Implements hook_field_prepare_view().
 */
function page2images_field_prepare_view($entity_type, $entities, $field, $instances, $langcode, &$items) {
  if ($field['type'] == 'page2images') {
    // If there are no files specified at all, use the default.
    foreach ($entities as $id => $entity) {
      if (empty($items[$id])) {
        $fid = 0;
        // Use the default for the instance if one is available.
        if (!empty($instances[$id]['settings']['wrapper']['default_image'])) {
          $fid = $instances[$id]['settings']['wrapper']['default_image'];
        }
        // Otherwise, use the default for the field.
        elseif (!empty($field['settings']['default_image'])) {
          $fid = $field['settings']['default_image'];
        }

        // Add the default image if one is found.
        if ($fid && ($file = file_load($fid))) {
          $items[$id][0] = (array) $file + array(
              'is_default' => TRUE,
              'alt' => '',
              'title' => '',
            );
        }
      }
    }
  }
}

/**
 * Implements hook_field_load().
 */
function page2images_field_load($entity_type, $entities, $field, $instances, $langcode, &$items, $age) {
  if ($field['type'] == 'page2images') {
    $fids = array();

    foreach ($entities as $id => $entity) {
      // Load the files from the files table.
      foreach ($items[$id] as $delta => $item) {
        if (isset($item['fid']) && (int) $item['fid'] > 0) {
          $fids[] = $item['fid'];
        }
      }
    }

    if (!empty($fids)) {
      $files = file_load_multiple($fids);
    }

    if (isset($files)) {
      foreach ($entities as $id => $entity) {
        foreach ($items[$id] as $delta => $item) {
          // If the file does not exist, mark the entire item as empty.
          if (!isset($item['fid']) || (int) $item['fid'] === 0 || !isset($files[$item['fid']])) {
            // $items[$id][$delta] = NULL;
          }
          else {
            $items[$id][$delta] = array_merge((array) $files[$item['fid']], $item);
          }
        }
      }
    }
  }
}

/**
 * Implements hook_field_presave().
 */
function page2images_field_presave($entity_type, $entity, $field, $instance, $langcode, &$items) {
  if ($field['type'] == 'page2images') {
    // If no image file, and token is not generated before.
    foreach ($items as $delta => $item) {
      if (isset($item['url']) && empty($items[$delta]['token']) && (int) $items[$delta]['fid'] === 0) {
        // Generate a token.
        $items[$delta]['token'] = _page2images_get_token($item['url']);

        $settings_defaults = $instance['settings'];
        $settings_saved = isset($instance['settings']['wrapper']) ? $instance['settings']['wrapper'] : array();

        $settings = array_merge($settings_defaults, $settings_saved);
        $settings['p2i_url'] = $item['url'];
        $settings['token'] = $items[$delta]['token'];
        $settings['entity_type'] = $entity_type;
        $settings['field'] = $field['field_name'];

        $result = page2images_call_p2i_with_callback($settings);

        if (!$result) {
          drupal_set_message(t('URL could not be send to Page2Images. Please check your error log.'), 'error');
        }
        else {
          drupal_set_message(t('Screenshot request has been sent to Page2Images.'), 'status');
        }
      }
    }
  }
}

/**
 * Implements hook_field_insert().
 */
function page2images_field_insert($entity_type, $entity, $field, $instance, $langcode, &$items) {
  if ($field['type'] == 'page2images') {
    list($id, $vid, $bundle) = entity_extract_ids($entity_type, $entity);

    // Add a new usage of each uploaded file.
    foreach ($items as $item) {
      if (isset($item['fid']) && (int) $item['fid'] > 0) {
        $file = (object) $item;
        file_usage_add($file, 'file', $entity_type, $id);
      }
    }
  }
}

/**
 * Implements hook_field_update().
 */
function page2images_field_update($entity_type, $entity, $field, $instance, $langcode, &$items) {
  if ($field['type'] == 'page2images') {
    list($id, $vid, $bundle) = entity_extract_ids($entity_type, $entity);

    // On new revisions, all files are considered to be a new usage and no
    // deletion of previous file usages are necessary.
    if (!empty($entity->revision)) {
      foreach ($items as $item) {
        if (isset($item['fid']) && (int) $item['fid'] > 0) {
          $file = (object) $item;
          file_usage_add($file, 'file', $entity_type, $id);
        }
      }
    } else {
      // Build a display of the current FIDs.
      $current_fids = array();
      foreach ($items as $item) {
        if (isset($item['fid']) && (int) $item['fid'] > 0) {
          $current_fids[] = $item['fid'];
        }
      }

      // Compare the original field values with the ones that are being saved. Use
      // $entity->original to check this when possible, but if it isn't available,
      // create a bare-bones entity and load its previous values instead.
      if (isset($entity->original)) {
        $original = $entity->original;
      }
      else {
        $original = entity_create_stub_entity($entity_type, array($id, $vid, $bundle));
        field_attach_load($entity_type, array($id => $original), FIELD_LOAD_CURRENT, array('field_id' => $field['id']));
      }

      $original_fids = array();
      if (!empty($original->{$field['field_name']}[$langcode])) {
        foreach ($original->{$field['field_name']}[$langcode] as $original_item) {
          if (isset($original_item['fid']) && (int) $original_item['fid'] > 0) {
            $original_fids[] = $original_item['fid'];
            if (isset($original_item['fid']) && !in_array($original_item['fid'], $current_fids)) {
              // Decrement the file usage count by 1 and delete the file if possible.
              file_field_delete_file($original_item, $field, $entity_type, $id);
            }
          }
        }
      }

      // Add new usage entries for newly added files.
      foreach ($items as $item) {
        if (isset($item['fid']) && (int) $item['fid'] > 0 && !in_array($item['fid'], $original_fids)) {
          $file = (object) $item;
          file_usage_add($file, 'file', $entity_type, $id);
        }
      }
    }
  }
}

/**
 * Implements hook_field_delete().
 */
function page2images_field_delete($entity_type, $entity, $field, $instance, $langcode, &$items) {
  list($id, $vid, $bundle) = entity_extract_ids($entity_type, $entity);

  // Delete all file usages within this entity.
  foreach ($items as $delta => $item) {
    if (isset($item['fid']) && (int) $item['fid'] > 0) {
      file_field_delete_file($item, $field, $entity_type, $id, 0);
    }
  }
}

/**
 * Implements hook_field_delete_revision().
 */
function page2images_field_delete_revision($entity_type, $entity, $field, $instance, $langcode, &$items) {
  list($id, $vid, $bundle) = entity_extract_ids($entity_type, $entity);

  foreach ($items as $delta => $item) {
    if (isset($item['fid']) && (int) $item['fid'] > 0) {
      // Decrement the file usage count by 1 and delete the file if possible.
      if (file_field_delete_file($item, $field, $entity_type, $id)) {
        $items[$delta] = NULL;
      }
    }
  }
}

/**
 * Implements hook_field_delete_field().
 */
function page2images_field_delete_field($field) {
  if ($field['type'] == 'page2images') {
    // The value of a managed_file element can be an array if #extended == TRUE.
    $fid = (is_array($field['settings']['default_image']) ? $field['settings']['default_image']['fid'] : $field['settings']['default_image']);
    if ($fid && ($file = file_load($fid))) {
      file_usage_delete($file, 'page2images', 'default_image', $field['id']);
    }
  }
}

/**
 * Implements hook_field_update_field().
 */
function page2images_field_update_field($field, $prior_field, $has_data) {
  if ($field['type'] != 'page2images') {
    return;
  }

  // The value of a managed_file element can be an array if #extended == TRUE.
  $fid_new = (is_array($field['settings']['default_image']) ? $field['settings']['default_image']['fid'] : $field['settings']['default_image']);
  $fid_old = (is_array($prior_field['settings']['default_image']) ? $prior_field['settings']['default_image']['fid'] : $prior_field['settings']['default_image']);

  $file_new = $fid_new ? file_load($fid_new) : FALSE;

  if ($fid_new != $fid_old) {

    // Is there a new file?
    if ($file_new) {
      $file_new->status = FILE_STATUS_PERMANENT;
      file_save($file_new);
      file_usage_add($file_new, 'page2images', 'default_image', $field['id']);
    }

    // Is there an old file?
    if ($fid_old && ($file_old = file_load($fid_old))) {
      file_usage_delete($file_old, 'page2images', 'default_image', $field['id']);
    }
  }

  // If the upload destination changed, then move the file.
  if ($file_new && (file_uri_scheme($file_new->uri) != $field['settings']['uri_scheme'])) {
    $directory = $field['settings']['uri_scheme'] . '://default_images/';
    file_prepare_directory($directory, FILE_CREATE_DIRECTORY);
    file_move($file_new, $directory . $file_new->filename);
  }
}

/**
 * Implements hook_field_delete_instance().
 */
function page2images_field_delete_instance($instance) {
  // Only act on image fields.
  $field = field_read_field($instance['field_name']);
  if ($field['type'] != 'page2images') {
    return;
  }

  // The value of a managed_file element can be an array if the #extended
  // property is set to TRUE.
  $fid = $instance['settings']['wrapper']['default_image'];
  if (is_array($fid)) {
    $fid = $fid['fid'];
  }

  // Remove the default image when the instance is deleted.
  if ($fid && ($file = file_load($fid))) {
    file_usage_delete($file, 'page2images', 'default_image', $instance['id']);
  }
}

/**
 * Implements hook_field_update_instance().
 */
function page2images_field_update_instance($instance, $prior_instance) {
  // Only act on image fields.
  $field = field_read_field($instance['field_name']);
  if ($field['type'] != 'page2images') {
    return;
  }

  // The value of a managed_file element can be an array if the #extended
  // property is set to TRUE.
  $fid_new = $instance['settings']['wrapper']['default_image'];
  if (is_array($fid_new)) {
    $fid_new = $fid_new['fid'];
  }
  $fid_old = $prior_instance['settings']['wrapper']['default_image'];
  if (is_array($fid_old)) {
    $fid_old = $fid_old['fid'];
  }

  // If the old and new files do not match, update the default accordingly.
  $file_new = $fid_new ? file_load($fid_new) : FALSE;
  if ($fid_new != $fid_old) {
    // Save the new file, if present.
    if ($file_new) {
      $file_new->status = FILE_STATUS_PERMANENT;
      file_save($file_new);
      file_usage_add($file_new, 'page2images', 'default_image', $instance['id']);
    }
    // Delete the old file, if present.
    if ($fid_old && ($file_old = file_load($fid_old))) {
      file_usage_delete($file_old, 'page2images', 'default_image', $instance['id']);
    }
  }

  // If the upload destination changed, then move the file.
  if ($file_new && (file_uri_scheme($file_new->uri) != $field['settings']['uri_scheme'])) {
    $directory = $field['settings']['uri_scheme'] . '://default_images/';
    file_prepare_directory($directory, FILE_CREATE_DIRECTORY);
    file_move($file_new, $directory . $file_new->filename);
  }
}

/**
 * Implements hook_field_widget_info().
 */
function page2images_field_widget_info() {
  return array(
    'page2images' => array(
      'label' => t('Default'),
      'field types' => array('page2images'),
      'behaviors' => array(
        'multiple values' => FIELD_BEHAVIOR_DEFAULT,
        'default value' => FIELD_BEHAVIOR_DEFAULT,
      ),
    ),
  );
}

/**
 * Implements hook_field_widget_form().
 */
function page2images_field_widget_form(&$form, &$form_state, $field, $instance, $langcode, $items, $delta, $element) {
  $required = $element['#required'];
  $item = &$items[$delta];

  $settings_defaults = $instance['settings'];
  $settings_saved = isset($instance['settings']['wrapper']) ? $instance['settings']['wrapper'] : array();
  $settings = array_merge($settings_defaults, $settings_saved);

  $element['fid'] = array(
    '#type' => 'hidden',
    '#value' => isset($item['fid']) ? $item['fid'] : 0,
  );

  $element['token'] = array(
    '#type' => 'hidden',
    '#value' => isset($item['token']) ? $item['token'] : '',
  );

  $element['width'] = array(
    '#type' => 'hidden',
    '#value' => isset($item['width']) ? $item['width'] : 0,
  );

  $element['height'] = array(
    '#type' => 'hidden',
    '#value' => isset($item['height']) ? $item['height'] : 0,
  );

  $element['url'] = array(
    '#title' => t('Page URL'),
    '#type' => 'textfield',
    '#required' => $required,
    '#default_value' => isset($item['url']) ? $item['url'] : '',
    '#description' => t('URL of target page. Do not need to be encode.'),
    '#element_validate' => array('_page2images_url_validate'),
    '#weight' => 1,
  );

  // Add the additional alt and title fields.
  $element['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => isset($item['title']) ? $item['title'] : '',
    '#description' => t('The title is used as a tool tip when the user hovers the mouse over the image.'),
    '#maxlength' => 1024,
    '#weight' => 1,
    '#access' => $settings['title_field'],
  );

  $element['alt'] = array(
    '#title' => t('Alternate text'),
    '#type' => 'textfield',
    '#default_value' => isset($item['alt']) ? $item['alt'] : '',
    '#description' => t('This text will be used by screen readers, search engines, or when the image cannot be loaded.'),
    '#maxlength' => 512,
    '#weight' => 2,
    '#access' => $settings['alt_field'],
  );

  return $element;
}

/**
 * Implements hook_field_widget_error().
 */
function page2images_field_widget_error($element, $error, $form, &$form_state) {
  if ($error['error_element']['url']) {
    form_error($element['url'], $error['message']);
  }
}

/**
 * Implements hook_field_formatter_info().
 */
function page2images_field_formatter_info() {
  return array(
    'page2images_rest' => array(
      'label' => t('Page2Images: Default'),
      'field types' => array('page2images'),
      'multiple values' => FIELD_BEHAVIOR_DEFAULT,
      'settings' => array(
        'url' => '',
        'image_style' => '',
        'image_link' => '',
      ),
    ),
  );
}

/**
 * Implements hook_field_formatter_settings_form().
 */
function page2images_field_formatter_settings_form($field, $instance, $view_mode, $form, &$form_state) {
  $display = $instance['display'][$view_mode];
  $settings = $display['settings'];

  $image_styles = image_style_options(FALSE, PASS_THROUGH);

  $element['image_style'] = array(
    '#title' => t('Image style'),
    '#type' => 'select',
    '#default_value' => $settings['image_style'],
    '#empty_option' => t('None (original image)'),
    '#options' => $image_styles,
  );

  $link_types = array(
    'content' => t('Content'),
    'website' => t('Website URL'),
    'file' => t('File'),
  );

  $element['image_link'] = array(
    '#title' => t('Link image to'),
    '#type' => 'select',
    '#default_value' => $settings['image_link'],
    '#empty_option' => t('Nothing'),
    '#options' => $link_types,
  );

  return $element;
}

/**
 * Implements hook_field_formatter_settings_summary().
 */
function page2images_field_formatter_settings_summary($field, $instance, $view_mode) {
  $display = $instance['display'][$view_mode];
  $settings = $display['settings'];

  $summary = array();

  $image_styles = image_style_options(FALSE, PASS_THROUGH);
  // Unset possible 'No defined styles' option.
  unset($image_styles['']);
  // Styles could be lost because of enabled/disabled modules that defines
  // their styles in code.
  if (isset($image_styles[$settings['image_style']])) {
    $summary[] = t('Image style: @style', array('@style' => $image_styles[$settings['image_style']]));
  }
  else {
    $summary[] = t('Original image');
  }

  $link_types = array(
    'content' => t('Linked to content'),
    'website' => t('Linked to Website URL'),
    'file' => t('Linked to file'),
  );
  // Display this setting only if image is linked.
  if (isset($link_types[$settings['image_link']])) {
    $summary[] = $link_types[$settings['image_link']];
  }

  return implode('<br />', $summary);
}

/**
 * Implements hook_field_formatter_view().
 */
function page2images_field_formatter_view($entity_type, $entity, $field, $instance, $langcode, $items, $display) {
  $element = array();

  $settings_defaults = $instance['settings'];
  $settings_saved = isset($instance['settings']['wrapper']) ? $instance['settings']['wrapper'] : array();
  $settings = array_merge($settings_defaults, $settings_saved);

  if ((int) $settings['default_image'] > 0) {
    $default_image = file_load($settings['default_image']);
  }

  // Check if the formatter involves a link.
  switch ($display['settings']['image_link']) {
    case 'content':
      $uri = entity_uri($entity_type, $entity);
      break;

    case 'website':
      $link_website = TRUE;
      break;

    case 'file':
      $link_file = TRUE;
      break;
  }

  foreach ($items as $delta => $item) {
    // Try to load the screenshot.
    $file = file_load($item['fid']);

    if ($file) {
      $item['uri'] = $file->uri;
    }
    elseif (isset($default_image->uri)) {
      $item['uri'] = $default_image->uri;
    }

    if (isset($link_website) && $link_website) {
      $uri = array(
        'path' => $item['url'],
        'options' => array(),
      );
    }

    if (isset($item['uri']) && isset($link_file) && $link_file) {
      $uri = array(
        'path' => file_create_url($item['uri']),
        'options' => array(),
      );
    }

    $element[$delta] = array(
      '#theme' => 'page2images_default_formatter',
      '#item' => $item,
      '#image_style' => $display['settings']['image_style'],
      '#path' => isset($uri) ? $uri : '',
    );
  }

  return $element;
}

/**
 * Element validate function for resolution fields.
 */
function _page2images_field_p2i_size_validate($element, &$form_state) {
  if (!empty($element['x']['#value']) || !empty($element['y']['#value'])) {
    foreach (array('x', 'y') as $dimension) {
      $value = $element[$dimension]['#value'];
      if (!is_numeric($value)) {
        form_error($element[$dimension], t('Height and width values must be numeric.'));
        return;
      }
    }
    form_set_value($element, intval($element['x']['#value']) . 'x' . intval($element['y']['#value']), $form_state);
  }
  else {
    form_set_value($element, '', $form_state);
  }
}

/**
 * Element validate function for resolution fields.
 */
function _page2images_field_p2i_screen_validate($element, &$form_state) {
  if (!empty($element['x']['#value']) || !empty($element['y']['#value'])) {
    foreach (array('x', 'y') as $dimension) {
      $value = $element[$dimension]['#value'];
      if (!is_numeric($value)) {
        form_error($element[$dimension], t('Height and width values must be numeric.'));
        return;
      }
      if (intval($value) == 0) {
        form_error($element[$dimension], t('Both a height and width value must be specified in the !name field.', array('!name' => $element['#title'])));
        return;
      }
    }
    form_set_value($element, intval($element['x']['#value']) . 'x' . intval($element['y']['#value']), $form_state);
  }
  else {
    form_set_value($element, '', $form_state);
  }
}

/**
 * Element validate callback for the file destination field.
 *
 * Remove slashes from the beginning and end of the destination value and ensure
 * that the file directory path is not included at the beginning of the value.
 */
function _page2images_file_directory_validate($element, &$form_state) {
  // Strip slashes from the beginning and end of $widget['file_directory'].
  $value = trim($element['#value'], '\\/');
  form_set_value($element, $value, $form_state);
}

/**
 * Element validate callback. Validates a URL.
 */
function _page2images_url_validate($element, &$form_state) {
  if (!empty($element['#value'])) {
    if (filter_var($element['#value'], FILTER_VALIDATE_URL)) {
      form_set_value($element, $element['#value'], $form_state);
    }
    else {
      form_error($element, t('Page URL is not a valid URL.'));
    }
  }
  else {
    form_set_value($element, '', $form_state);
  }
}
